/**********************************************************************/
/** Copyright (C) 2021 - Université Catholique de Louvain		**/
/** secantmfh version 2021.1						**/
/** Author: Mohamed HADDAD (immc-MEMA)				**/
/** Contact: mohamed.haddad@uclouvain.be				**/
/** Use of this source code is governed by an UCL-style license	**/ 
/** All Rights Reserved						**/
/** This project has received funding from the European 		**/
/** Union’s Horizon 2020 research and innovation program		**/
/** under grant agreement No 862015					**/
/***********************************************************************/


#ifndef FONCTIONS_PRATIQUES_H
#define FONCTIONS_PRATIQUES_H 1


/* Convert a 2nd order tensor 3*3 to a vector 6*1 (Issam's notation) */
void convertmatrvect (double **matr, double *vect);

/* Convert a vector 6*1 to matrix 3*3 (Issam's notation) */
void convertvectmatr (double *vect, double **matr);

/* Addition of two tensors of the fourth order, each one can be previously multiplied by a scalar */
void addtens4 (double a, double **A, double b, double **B, double **res);

/* Addition of two tensors of the second order, each one can be previously multiplied by a scalar */
void addtens2 (double a, double *A, double b, double *B, double *res);

/* Contraction of two 2nd order tensor under the form of vectors 6*1 */
double contraction22 (const double *, const double *);

/* Contraction of 4th order tensor (6*6) with a 2nd order tensor under the form of vector 6*1 */
void contraction42 (double **T4, double *T2, double *Sol);

/* Contraction of two 4th order tensor (6*6) */
void contraction44 (double **, double **, double **);

/* Double contraction of two 4th order tensor (6*6) */
double dcontr44 (double **TA, double **TB);

/* Double contraction of three fourth order tensors; last contraction being over four indices */
double contraction444 (double **A, double **B, double **C);

/* Double contraction of a two fourth order tensors and one of the second order */
void contraction442 (double **A, double **B, double *C, double *Tres);

/* Contraction of three 4th order tensor - same two first indices of the first tensor - idem for the two last ones of the last tensor */
double contr444_reduced (double **A, double **B, double **C);

/* Produit vectoriel de deux tenseurs d'ordre2(6*1) */
void produit_tensoriel (double *TA, double *TB, double **Tres);

/* Compute the rotation matrix q which transform a Sd order tensor from the local to the global system of axes */
void compute_q (double phi, double theta, double psi, double **q);

/* Changement de repere d'un tenseur d'ordre 2 du repere global au repere local */
void rotatetens2gloloc(double *, double, double, double, double *);

/* Changement de repere d'un tenseur d'ordre 2 du repere local au repere global */
void rotatetens2locglo (double *tenseur, double phi, double theta, double psi, double *tenseur_rot);

/* Changement de repere d'un tenseur d'ordre 4 du repere global au repere local */
void rotatetens4gloloc (double **, double, double, double, double **);

/* Changement de repere d'un tenseur d'ordre 4 du repere local au repere global */
void rotatetens4locglo (double **tenseur, double phi, double theta, double psi, double **tenseur_rot);

/* Factorisatin LU d'une matrice d'ordre n */
void ludcmp(double **a, int n, int *indx, double *d, int *error);

/* Forward and backward substitution */
void lubksb (double **a, int n, int *indx, double b[]);

/* Computes the inverse of a matrix of size n */
void inverse (double **a, double **y, int *error, int n);

/* Extract isotropic part of a fourth order tensor */
void extractiso (double **tensani, double **tensiso);

/* Memory allocation of a table of pointers */
void mallocmatrix (double ***A, int m, int n);

/* Freeing memory of a table of pointers */
void freematrix (double **A, int m);

/* Memory allocation for a 3D matrix */
void mallocmatrix3D (double ****A, int p, int m, int n);

/* Freeing memory of a 3D matrix */
void freematrix3D (double ***A, int p, int m);

/* Compute the transverse isotropic Hooke's elastic operator from E and nu */
void compute_Hooke(double E, double nu, double **C);

/* Print on the screen values of a vector */
void printvect(double *vect,int  n);

/* Print values of a vector to a specified file */
void fprintvect(FILE *pfile, double *vect, int n);

/* Print on the screen values of a m*n matrix */
void printmatr(double **matr4, int m, int n);

/* Print values of a m*n matrix to a specified file */
void fprintmatr(FILE *pfile, double **matr4, int m, int n);

/* Compute the trace of a second order tensor */
double trace(double *tens2);

/* Extract the deviatoric part of a second order tensor */
void dev(double *tens2, double *dev_tens2);

/* Compute the von Mises equivalent stress */
double vonmises(double *tens2);

/* Compute the equivalent strain scalar */
double eqstrn(double *tens2);

/* Compare two vectors */
int compare(double *A, double *B, int size);

/* Compare two matrices */
int comparematr(double **A, double **B, int m, int n);

/* Check if a vector is null */
int isnull(double *A, int size);

/* Check if a fourth order tensor is isotropic */
int isisotropic(double**A);

/* Copy a vector into another */
void copyvect(double *A, double *B, int size);

/* Find the maximum value of a vector */
double maxvect(double *vect, int n);

/* Find the maximum value in absolute value of a vector */
double maxvectabs(double *vect, int n);

/* Find the minimum non nil value in absolute value of a vector */
double minvectabs(double *vect, int m);

/* Find the smallest non-nil value in absolute value of a matrix */
double minmatrabs(double **matr, int m, int n);

/* Find max of two values */
double max(double a, double b);


/* Compute I */
void compute_I(double **I);

/* Compute Ivol */
void compute_Ivol(double **I_vol);

/* Compute Idev */
void compute_Idev(double **I_dev);

/* Compute zero vector of size size **/
void zero_vect(double size, double *vect);

/* A(6*1) : I(6*6) : B(6*1)  */
double contraction242(double *A, double **I, double*B);

#endif
