import os
import sys
from gmshpy import *
from dG3Dpy import*
from math import*
import numpy as np
import pandas as pd

step_toll = 1000 #10000

dataFile = sys.argv[1]
meshfile = sys.argv[2]


print('start running: '+dataFile)
data = pd.read_csv(dataFile, sep=';', dtype=float)
strainPath = data.values
N = len(strainPath) # Total Data points in Load path including 0
print('strainPath',strainPath)

mat = open("MaterialProperties.csv", 'r')
params = mat.readline().split(';')

rho = float(params[0])
E = float(params[1]) 
nu = float(params[2]) 
g1 = float(params[3]) 
k1 = float(params[4]) 
kk1 = float(params[5])
kk2 = float(params[6])
kk3 = float(params[7])
kk4 = float(params[8])
kk5 = float(params[9])
kk6 = float(params[10])
kk7 = float(params[11])
kk8 = float(params[12])
gg1 = float(params[13])
gg2 = float(params[14])
gg3 = float(params[15])
gg4 = float(params[16])
gg5 = float(params[17])
gg6 = float(params[18])
gg7 = float(params[19])
gg8 = float(params[20])
eta = float(params[21])
s = float(params[22])
alpha = float(params[23])
beta = 1.5*float(params[24]) 
sy0c = float(params[25])
c0 = float(params[26])
hhc0 = float(params[27])
hc = float(params[28])
m = float(params[29])
sy0t = float(params[30])
t0 = float(params[31])
hht0 = float(params[32])
ht = float(params[33])

mat.close()

# material law
lawnum1 = 11 # unique number of law

hardenc = LinearExponentialJ2IsotropicHardening(1, sy0c, c0, hhc0, hc)
hardent = LinearExponentialJ2IsotropicHardening(2, sy0t, t0, hht0, ht)
#hardenk = PolynomialKinematicHardening(3,1)
#hardenk.setCoefficients(1,370.)

law1   = HyperViscoElastoPlasticPowerYieldDG3DMaterialLaw(lawnum1,rho,E,nu,1e-4)#,False,1e-8)
law1.setCompressionHardening(hardenc)
law1.setTractionHardening(hardent)
#law1.setKinematicHardening(hardenk)

law1.setStrainOrder(5)

law1.setYieldPowerFactor(alpha)
law1.setNonAssociatedFlow(True)
law1.nonAssociatedFlowRuleFactor(beta)

etac = constantViscosityLaw(1,eta)
law1.setViscosityEffect(etac,s)


law1.setViscoelasticMethod(0)

law1.setViscoElasticNumberOfElement(8)

law1.setViscoElasticData_Bulk(1, kk1, k1)
law1.setViscoElasticData_Bulk(2, kk2, k1*0.1)
law1.setViscoElasticData_Bulk(3, kk3, k1*0.01)
law1.setViscoElasticData_Bulk(4, kk4, k1*0.001)
law1.setViscoElasticData_Bulk(5, kk5, k1*0.0001)
law1.setViscoElasticData_Bulk(6, kk6, k1*0.00001)
law1.setViscoElasticData_Bulk(7, kk7, k1*0.000001)
law1.setViscoElasticData_Bulk(8, kk8, k1*0.0000001)

law1.setViscoElasticData_Shear(1, gg1, g1)
law1.setViscoElasticData_Shear(2, gg2, g1*0.1)
law1.setViscoElasticData_Shear(3, gg3, g1*0.01)
law1.setViscoElasticData_Shear(4, gg4, g1*0.001)
law1.setViscoElasticData_Shear(5, gg5, g1*0.0001)
law1.setViscoElasticData_Shear(6, gg6, g1*0.00001)
law1.setViscoElasticData_Shear(7, gg7, g1*0.000001)
law1.setViscoElasticData_Shear(8, gg8, g1*0.0000001)

# geometry
meshfile = meshfile # name of mesh file
Mfield = 51
myfield1 = dG3DDomain(1000,Mfield,0,lawnum1,0,3,0)

# solver
sol = 2  # Gmm=0 (default) Taucs=1 PETsc=2
soltype =1 # StaticLinear=0 (default) StaticNonLinear=1
nstep = N-1  # number of step (used only if soltype=1)
ftime =strainPath[N-1][7]  # Final time (used only if soltype=1)
tol=1.e-5  # relative tolerance for NR scheme (used only if soltype=1)
nstepArch = int(nstep*0.01) #2*N # Number of step between 2 archiving (used only if soltype=1)


# creation of Solver
mysolver = nonLinearMechSolver(1000)
#mysolver.createModel(Geo,meshfile,3,2)
mysolver.loadModel(meshfile)
mysolver.addDomain(myfield1)
mysolver.addMaterialLaw(law1)
mysolver.Scheme(soltype)
mysolver.Solver(sol)
mysolver.snlData(nstep,ftime,tol,tol/100.)
mysolver.stepBetweenArchiving(nstepArch)
mysolver.pathFollowing(False,0)
mysolver.setPathFollowingControlType(1)
mysolver.setPathFollowingArcLengthStep(1e-4)


timePlan = PointwiseTimeManager(1.e-12,1.e-10)
print("N: ", N)

for i in range(N-1):

  U0 = strainPath[i][0:6]
  U1 = strainPath[i+1][0:6]
  stepSize = int(np.max(np.abs(np.asarray(U1) - np.asarray(U0)))*step_toll)
  
  if stepSize < 5:
     stepSize = 5
  
  srcor=int(np.log10(strainPath[i+1][9]/1.e-6))+1
  if(srcor<1):
    srcor=1
  print("step size: ", strainPath[i+1][9], srcor,stepSize)
  stepSize=stepSize #*srcor

  timePlan.setNumStepTimeInterval(strainPath[i+1][7],stepSize)
  timePlan.put(strainPath[i+1][7])

mysolver.setTimeManager(timePlan)
mysolver.snlManageTimeStep(50,2,2.,500)

system = 1   # Displacement elimination =0 Multiplier elimination = 1 Displacement+ multiplier = 2
control = 0     # load control = 0 arc length control euler = 1
mysolver.setSystemType(system)
mysolver.setControlType(control)
mysolver.options("-ksp_type preonly -pc_type lu -pc_factor_mat_solver_type petsc")

mysolver.setMessageView(True)

microBC = nonLinearPeriodicBC(1000,3)
microBC.setOrder(1)
microBC.setBCPhysical(100,110,120,101,111,121)
method = 0	#[0] Periodic mesh = 0
degree = 3 # [3]Order used for polynomial interpolation
addvertex = 0  # Polynomial interpolation by mesh vertex = 0, Polynomial interpolation by virtual vertex
microBC.setPeriodicBCOptions(method, degree, bool(addvertex))
microBC.setDeformationGradient(2.,1.,1.,1.,2.,1.,1.,1.,2.)


fctxx = piecewiseScalarFunction()
fctyy = piecewiseScalarFunction()
fctzz = piecewiseScalarFunction()
fctxy = piecewiseScalarFunction()
fctyx = piecewiseScalarFunction()
fctxz = piecewiseScalarFunction()
fctzx = piecewiseScalarFunction()
fctyz = piecewiseScalarFunction()
fctzy = piecewiseScalarFunction()

for i in range(N):
  fctxx.put(strainPath[i][7] ,strainPath[i][0])
  fctyy.put(strainPath[i][7] ,strainPath[i][1])
  fctzz.put(strainPath[i][7] ,strainPath[i][2])
  fctxy.put(strainPath[i][7] ,strainPath[i][3])
  fctyx.put(strainPath[i][7] ,strainPath[i][3])
  fctxz.put(strainPath[i][7] ,strainPath[i][4])
  fctzx.put(strainPath[i][7] ,strainPath[i][4])
  fctyz.put(strainPath[i][7] ,strainPath[i][5])
  fctzy.put(strainPath[i][7] ,strainPath[i][5])


microBC.setPathFunctionDeformationGradient(0,0,fctxx)
microBC.setPathFunctionDeformationGradient(1,1,fctyy)
microBC.setPathFunctionDeformationGradient(2,2,fctzz)
microBC.setPathFunctionDeformationGradient(0,1,fctxy)
microBC.setPathFunctionDeformationGradient(1,0,fctyx)
microBC.setPathFunctionDeformationGradient(0,2,fctxz)
microBC.setPathFunctionDeformationGradient(2,0,fctzx)
microBC.setPathFunctionDeformationGradient(1,2,fctyz)
microBC.setPathFunctionDeformationGradient(2,1,fctzy)

mysolver.addMicroBC(microBC)

mysolver.stressAveragingFlag(True) # set stress averaging ON- 0 , OFF-1
mysolver.setStressAveragingMethod(0) # 0 -volume 1- surface
#tangent averaging flag
mysolver.tangentAveragingFlag(False) # set tangent averaging ON -0, OFF -1 #** If turned off, Elast Tensor will not be computed
mysolver.setTangentAveragingMethod(2,1e-6) # 0- perturbation 1- condensation

# build view
mysolver.internalPointBuildView("Strain_xx",IPField.STRAIN_XX, 1, 1);
mysolver.internalPointBuildView("Strain_yy",IPField.STRAIN_YY, 1, 1);
mysolver.internalPointBuildView("Strain_zz",IPField.STRAIN_ZZ, 1, 1);
mysolver.internalPointBuildView("Strain_xy",IPField.STRAIN_XY, 1, 1);
mysolver.internalPointBuildView("Strain_yz",IPField.STRAIN_YZ, 1, 1);
mysolver.internalPointBuildView("Strain_xz",IPField.STRAIN_XZ, 1, 1);
mysolver.internalPointBuildView("sig_xx",IPField.SIG_XX, 1, 1);
mysolver.internalPointBuildView("sig_yy",IPField.SIG_YY, 1, 1);
mysolver.internalPointBuildView("sig_zz",IPField.SIG_ZZ, 1, 1);
mysolver.internalPointBuildView("sig_xy",IPField.SIG_XY, 1, 1);
mysolver.internalPointBuildView("sig_yz",IPField.SIG_YZ, 1, 1);
mysolver.internalPointBuildView("sig_xz",IPField.SIG_XZ, 1, 1);
mysolver.internalPointBuildView("sig_VM",IPField.SVM, 1, 1);
'''
mysolver.internalPointBuildView("Equivalent plastic strain",IPField.PLASTICSTRAIN, 1, 1);
mysolver.internalPointBuildView("pression",IPField.PRESSION,1,1)
mysolver.internalPointBuildView("plastic possion ratio",IPField.PLASTIC_POISSON_RATIO,1,1)

mysolver.internalPointBuildView("ISO_YIELD",IPField.ISO_YIELD,1,1)

mysolver.internalPointBuildView("Ee_ZZ",IPField.Ee_ZZ,1,1)
mysolver.internalPointBuildView("Ee_YY",IPField.Ee_YY,1,1)
mysolver.internalPointBuildView("Ee_XX",IPField.Ee_XX,1,1)

mysolver.internalPointBuildView("Eve1_ZZ",IPField.Eve1_ZZ,1,1)
mysolver.internalPointBuildView("Eve1_YY",IPField.Eve1_YY,1,1)
mysolver.internalPointBuildView("Eve1_XX",IPField.Eve1_XX,1,1)
'''

#archiving
mysolver.archivingIPDataOnPhysical(IPField.P_ZZ,Mfield,3,1,True,-1)
mysolver.archivingAverageValue(IPField.P_XX)
mysolver.archivingAverageValue(IPField.P_YY)
mysolver.archivingAverageValue(IPField.P_ZZ)
mysolver.archivingAverageValue(IPField.P_XY)
mysolver.archivingAverageValue(IPField.P_YX)
mysolver.archivingAverageValue(IPField.P_XZ)
mysolver.archivingAverageValue(IPField.P_ZX)
mysolver.archivingAverageValue(IPField.P_YZ)
mysolver.archivingAverageValue(IPField.P_ZY)

mysolver.archivingAverageValue(IPField.F_XX)
mysolver.archivingAverageValue(IPField.F_YY)
mysolver.archivingAverageValue(IPField.F_ZZ)
mysolver.archivingAverageValue(IPField.F_XY)
mysolver.archivingAverageValue(IPField.F_YX)
mysolver.archivingAverageValue(IPField.F_XZ)
mysolver.archivingAverageValue(IPField.F_ZX)
mysolver.archivingAverageValue(IPField.F_YZ)
mysolver.archivingAverageValue(IPField.F_ZY)

internalVars = [IPField.SVM]
for i in internalVars:
  mysolver.archivingAverageValueOnPhysical(51,i)
# solve
mysolver.solve()

print('done running: ' + dataFile)

