#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
import glob
import sys
import os,shutil, fnmatch, pickle
from sys import exit
import matplotlib.pyplot as plt
from scipy.spatial import distance

import utilRF

listTest=['cellRF','rnnRF']
testNb=1
test= listTest[testNb]

Keys=[]
if test=='cellRF':
  mechDir='../cellRF' # mechanical folder and file
  Keys=['Porosity']
elif test=='rnnRF':
  mechDir='../rnnRF' # mechanical folder and file
  Keys=['Radius','Length']

rfDir=mechDir+'/randomFields'
gpInfoDir=mechDir+'/GPData'

# read parameters
with open(rfDir+'/input.dat','rb') as data:
    Nsim=pickle.load(data)
    nVar=pickle.load(data)
    nDim=pickle.load(data)
    gPTetNb=pickle.load(data)
    gPNb=pickle.load(data)
    x=pickle.load(data)

#Nsim=6
if len(Keys)!=nVar:
  Keys=[]
  for i in range(nVar):
    Keys.append('var_'+str(i))


for i in range(Nsim):
  rffilename=rfDir+'/RandField_'+str(i)+'.csv'
  print("read RF file ",rffilename)  
  outname=rfDir+'/RandField_'+str(i)+'.msh'
  randf = utilRF.RFReadOneRandField(rffilename,nVar,len(x))
  print("Write RF under mesh form",outname)
  utilRF.writeMeshFile(gpInfoDir,outname,i+1,gPTetNb,Keys,randf)
'''
# reading vector of random field to *.csv files
fpath2 = './fileRandField/DogBone/'
startnum=0  #number to initialise RF file
fname2 = lambda m: "RandField" + "_chap_" + str(m+startnum) + ".csv"

# reading *.csv files from mechanical model
fpath = "../FatigueDamage/testGFRP/RH50RF/R0XDir/"         # folder path
path_out = fpath                  #file with disp and stress files
outlist = ["RandFields_DogBone_DataFrame"]

#writting RF at Gauss points 
fname3 = lambda m: "RandFieldAtGP" + "_chap_" + str(m+startnum) + ".csv"
fileout = lambda n,rf: "/IPDdata_" + str(outlist[n])+"RFnb_"+str(rf+startnum) + "_OnPhysical_83" + ".msh"   # path to write random fields


varlist = ['SVM']   # list of input components
fname = lambda m: "/coordinate_" + str(varlist[m]) + "_OnPhysical_83" + ".csv"

main_df = SubFunctions.ReadMechDataList(fpath,fname,varlist)




with open(fpath2+'input.dat','rb') as data:
    Nsim=pickle.load(data)
    nVar=pickle.load(data)
    nDim=pickle.load(data)
    nCoord=pickle.load(data)
    nCXYZ=pickle.load(data)
    xlow=pickle.load(data)
    xupp=pickle.load(data)
    dx=pickle.load(data)
    x=pickle.load(data)
    x_min=pickle.load(data)
    x_max=pickle.load(data)

#Nsim=100  #can tune Nsim
#==============================================================================
# COMPUTATIONAL DOMAIN
#==============================================================================
# Export stress components
ngpe = 4                            # no. Gauss points per element
nTet = int(len(main_df[0])/ngpe)     # no. elements
nGTotal = nTet*ngpe                   # no. total Gauss points


# extracting Gauss points' coordinates
iTet = main_df[0][0:nGTotal:ngpe,0]   # vector of element ID
iTet = iTet - iTet[0]      # the 1st element does not start from 1 = ()
xG = main_df[0][0:nGTotal,1]      # x-corrd. of the Gauss point = (nGTotal,1)
yG = main_df[0][0:nGTotal,2]      # y-coord. of the Gauss point = (nGTotal,1)
zG = main_df[0][0:nGTotal,3]      # z-coord. of the Gauss point = (nGTotal,1)
gCoord = main_df[0][0:nGTotal,[1, 2, 3]]    # x,y,z coordinates = (nGTotal,3)
iGauss = main_df[0][0:nGTotal,4]     # Gauss' ID of the elements = (nGTotal,1)
# reordering the Gauss point indices
iGTet = np.zeros((nTet,ngpe)).astype(int)
for iel in range(nTet):
    iGTet[iel,:] = iGauss[0:ngpe] + iel*ngpe
#iGTet = np.reshape(iGauss,(nTet,ngpe)).astype(int)   # (nTet,ngpe)

# determine min, max coordinates
gCoordMin = np.min(gCoord,0)
gCoordMax = np.max(gCoord,0)

"""
#==============================================================================
# plotting Gauss points in computational domains
RFCallDataFiles.RFPlotGaussPoints(xG,yG,zG)
"""

#==============================================================================
# RANDOM FIELD DOMAIN
#==============================================================================

xMin = np.min(x,axis=0)
xMax = np.max(x,axis=0)
print("Grid bounds: ",xMin,xMax)
print("GP location bounds: ",gCoordMin,gCoordMax)
#exit()
#==============================================================================
# reading the numerical correlation matrix
# reading the numerical correlation matrix
#fpath1 = './fileCorrMat/DogBone/'
#fname1 = lambda m,n: "NumCorrMat" + "_chap_" + str(m) + "_page_" + str(n) + ".csv"
#Rshape = np.array([nVar,nVar,nCXYZ,nCXYZ])   # sizes of the numerical correlation matrix = (4,1)
#Rnum_XY = RFWriteReadData.RFReadNumCorrMat(fpath1,fname1,Rshape)    # (nVar,nVar,nCXYZ,nCXYZ)

#==============================================================================
numx = nCoord[0] - 1      # no. cubes in x-dir
numy = nCoord[1] - 1
numz = nCoord[2] - 1
ncubes = numx*numy*numz     # total no. cubes

# nodal connectivity for the 1st element
nCXY=nCoord[0]*nCoord[1]
ndpatern = np.array([[0, 1, nCoord[0]+1, nCoord[0], nCXY, nCXY+1, nCXY+nCoord[0]+1, nCXY+nCoord[0]]])   # (1,8)
# cube = (nTet,8)
# cbCoord = (nnode,nDim)    # nnode = no. nodes per element
cube, cbCoord = RFSubFuncs.RFNodalConnectivity(x,numx,numy,numz,nCoord[0],nCoord[1],nCoord[2],ndpatern,nDim)

"""
#inc = np.zeros((1, ndpatern.shape[0]))
inc = 0
e = -1
cube = np.zeros((ncubes, ndpatern.shape[1])).astype(int) # (nTet,8)
cbCoord = [np.zeros((ndpatern.shape[1],nDim)) for i in range(ncubes)]  # nodal coord. of a cube
for iz in range(1,numz+1):
    for iy in range(1,numy+1):
        for ix in range(1,numx+1):
            inc = (iz-1)*nCoord[0]*nCoord[1] + (iy-1)*nCoord[0] + (ix-1)
            e = e + 1
            # connectivity of the current cube
            cube[e,:] = ndpatern + inc   # (8,1)
            # nodal coordinates of the current cube
            cbCoord[e] = x[cube[e,:],:]   # cbCoord[i] = (nnode,nDim)
"""

"""
##check = RFSubFuncs.TestPntInsideCube(points, cbCoord)        # list( set().union(res1, res2, res3) )
#==============================================================================
# check if a Gauss point is within a hexagonal element
Rnum = np.zeros((nVar,nVar,nGTotal,nGTotal))        # corr. values at Gauss points
check = [[0 for j in range(nGTotal)] for i in range(ncubes)]
ptlist_cube = [[] for i in range(ncubes)]    # initialize a 2D list = [ncubes][...]
indlist_cube = [[] for i in range(ncubes)]    # initialize a 2D list = [ncubes][...]
for icb in range(ncubes):    # loops on the no. cube elements = 27
    for igp in range(nGTotal):    # loops on total no. Gauss points = 720
        ptTest = gCoord[igp,:].reshape(1,-1)    # coordinates of Gauss points = (1,nDim)
        check[icb][igp] = RFSubFuncs.TestPntInsideCube(ptTest, cbCoord[icb])      # checking
        
        if check[icb][igp] == 1:
            # print("Point", ptTest, "Cube", cbCoord[icb])
            # distances between a Gauss point and RF grid points = (8,1)
            dist = distance.cdist(cbCoord[icb], ptTest, 'euclidean')  # cbCoord[i] = (8,3); pt2D = (1,3)
            indMin = np.argmin(dist)        # min index of the distance vector
            ptlist_cube[icb].append(igp)    # add the accepted Gauss point inside the current cube into list
            indlist_cube[icb].append(cube[icb,indMin])    # add the closest node into the list
            
            Rnum[0,0,0,igp] = Rnum_XY[0,0,0,cube[icb,indMin]]
            Rnum[0,1,0,igp] = Rnum_XY[0,1,0,cube[icb,indMin]]
            Rnum[1,0,0,igp] = Rnum_XY[1,0,0,cube[icb,indMin]]
            Rnum[1,1,0,igp] = Rnum_XY[1,1,0,cube[icb,indMin]]

#==============================================================================
count = 0
ptlist_check = []
for i in range(ncubes):
    ptlist_check.extend(ptlist_cube[i]) # accumulate the points from lists
    out = len(ptlist_cube[i])
    count = count+out
    
#ptlist_check.sort()
print("The number of points counted is", count)
"""
#==============================================================================

# list of files containing matrix (2*ny-1,2*nz-1)
flist2 = [0 for i in range(Nsim)]     # initialize 2D list
for i in range(Nsim):
    flist2[i] = glob.glob(fpath2 + fname2(i))

# flatten 2D list into 1D list
flist2 = [val for slist in flist2 for val in slist]     # list[nDtau](2*nx-1,) --> considering zero padding

# list contains list of matrix (2*ny-1,2*nz-1)
output = [np.zeros((nCXYZ,nVar)) for i in range(Nsim)]  # [Nsim](nCXYZ,nVar)
for i in range(Nsim):
    data = pd.read_csv(flist2[i], delimiter='; ', header=None, engine = 'python') # (2*ny-1,2*nz-1)
    df = data.to_numpy()         # array (2*ny-1,2*nz-1)
    print("chapter %2d" % (i))
    output[i] = df

"""
#==============================================================================
# assigning random fields to the Gauss points
# check if a Gauss point is within a hexagonal element
randf1 = np.zeros((Nsim,nGTotal))       # the 1st random field at Gauss points
randf2 = np.zeros((Nsim,nGTotal))       # the 2nd random field at Gauss points
check = [[0 for j in range(nGTotal)] for i in range(ncubes)]
ptlist_cube = [[] for i in range(ncubes)]    # initialize a 2D list
indlist_cube = [[] for i in range(ncubes)]    # initialize a 2D list
for icb in range(ncubes):    # loops on the no. cube elements = 27
    for igp in range(nGTotal):    # loops on total no. Gauss points = 720
        ptTest = gCoord[igp,:].reshape(1,-1)    # coordinates of Gauss points = (1,nDim)
        check[icb][igp] = RFSubFuncs.TestPntInsideCube(ptTest, cbCoord[icb])      # checking
        
        if check[icb][igp] == 1:
            # print("point detected")
            # distances between a Gauss point and RF grid points = (8,1)
            dist = distance.cdist(cbCoord[icb], ptTest, 'euclidean')  # cbCoord[i] = (8,3); pt2D = (1,3)
            indMin = np.argmin(dist)        # min index of the distance vector
            ptlist_cube[icb].append(igp)    # add the accepted point inside the current cube into list
            indlist_cube[icb].append(cube[icb,indMin])    # add the closest node into the list
            
            randf1[0,igp] = output[0][cube[icb,indMin],0]      # for the 1st element
            randf2[0,igp] = output[0][cube[icb,indMin],1]      # for the 2nd element
"""
# assigning random fields to the Gauss points
tmprf=np.zeros((nVar,nGTotal))
for iReal in range(Nsim):
    randf = np.zeros((nVar,nGTotal))       # the random fields at Gauss points
    check = [[0 for j in range(nGTotal)] for i in range(ncubes)]
    ptlist_cube = [[] for i in range(ncubes)]    # initialize a 2D list
    indlist_cube = [[] for i in range(ncubes)]    # initialize a 2D list
    for icb in range(ncubes):
        for igp in range(nGTotal):    # loops on total no. Gauss points = 720
            ptTest = gCoord[igp,:].reshape(1,-1)    # coordinates of Gauss points = (1,nDim)
            # check if a Gauss point is within a hexagonal element
            check[icb][igp] = RFSubFuncs.RFTestPtInsideCube(ptTest, cbCoord[icb])      # checking
            
            if check[icb][igp] == 1:
                # print("point detected")
                # distances between a Gauss point and RF grid points = (8,1)
                dist = distance.cdist(cbCoord[icb], ptTest, 'euclidean')  # cbCoord[i] = (8,3); pt2D = (1,3)
                indMin = np.argmin(dist)        # min index of the distance vector
                ptlist_cube[icb].append(igp)    # add the accepted point inside the current cube into list
                indlist_cube[icb].append(cube[icb,indMin])    # add the closest node into the list
                
                #randf1[0,igp] = output[0][cube[icb,indMin],0]      # for the 1st element
                #randf2[0,igp] = output[0][cube[icb,indMin],1]      # for the 2nd element
                for var in range(nVar):
                  randf[var, igp] = output[iReal][cube[icb,indMin],var]      #
    RFWriteReadData.RFWriteOneRandField(fpath,fname3(iReal),randf,nVar)        

#==============================================================================
# average the random fields at the element centroids
#randf_ave1 = RFSubFuncs.RFCenAverage(nTet,nGTotal,ngpe,randf1,iGTet,iReal)     # (nTet,1)
#randf_ave2 = RFSubFuncs.RFCenAverage(nTet,nGTotal,ngpe,randf2,iGTet,iReal)     # (nTet,1)

#==============================================================================
# COMPUTE AND OUTPUT THE AVERAGED RANDOM FIELD TO *.MSH FILE
#==============================================================================
fpstress = lambda m: fpath + "/stress_step" + str(m) + ".msh"     # path to read stress info.
nstep = 1
inname=fpstress(nstep)

for iReal in range(Nsim):
  outname=path_out + fileout(0,iReal)
  randf = RFWriteReadData.RFReadOneRandField(fpath,fname3(iReal),nVar,nGTotal)
  Keys = ["Svm0", "b0"]
  print("Write ",outname)
  WriteCSVOutputs.writeMeshFile(inname,outname,nstep,nGTotal,ngpe,Keys,randf)

'''
